﻿Imports MySql.Data.MySqlClient
Imports System.Data

Namespace TechNote31_MySQLClient_VB
    ' 데이터베이스 처리용 사원 자료 구조
    Public Structure Employee
        Public strName As String        ' 성명
        Public iAge As Integer          ' 나이
        Public strPosition As String    ' 직위
        Public strPart As String        ' 부서
        Public dDatetime As DateTime    ' 입사일
    End Structure

    Public Class CCEADODOTNET
        ' 데이터베이스 커넥션 객체 선언
        Private m_conn As MySqlConnection
        ' 쿼리 객체 생성. 데이터의 삽입, 삭제
        ' 갱신을 위해 MySqlCommand 선언
        Private comm As MySqlCommand

        ' 데이터베이스 연결 처리 (성공 : 리턴값 True, 실패 : 리턴값 False)
        Public Function Connection(ByVal strServerAddress As String, ByVal strDBName As String, ByVal strUserID As String, ByVal strPassword As String) As Boolean
            Dim strConnection As String

            ' strServerAddress 는 MySQL Server의 주소를 입력하며 도메인 혹은 IP를 입력합니다.
            ' strUserID 는 MySQL Server에 접속하는 경우 연결할 UserID를 입력합니다. 
            ' strPassword 는 해당 UserID의 Password를 입력합니다. 
            ' StrDBName 는 해당 DBName을 입력하며 MySQLServer에서 생성된 DB명을 입력합니다. 

            strConnection = String.Format("server={0};uid={1};pwd={2};database={3};Charset=utf8", strServerAddress, strUserID, strPassword, strDBName)

            Try
                ' 데이터베이스 연결 설정
                m_conn = New MySqlConnection(strConnection)

                ' 데이터베이스 연결
                m_conn.Open()

                ' 데이터베이스 smartx에 EMPLOYEE 테이블이 존재하는 경우 삭제하고 다시 만들어 줍니다. 
                ' 데이터베이스 연결시 최초 1회만 실행합니다. 
                Dim sql As String = "DROP TABLE IF EXISTS EMPLOYEE; CREATE TABLE EMPLOYEE(NAME varchar(30), AGE int(5), POSITION varchar(30), PART varchar(30), ENTERING DateTime)"
                ' Dim sql As String = "CREATE TABLE IF NOT EXISTS EMPLOYEE (NAME varchar(30), AGE int(5), POSITION varchar(30), PART varchar(30), ENTERING DateTime)"
                comm = New MySqlCommand(sql, m_conn)

                ' 연결에 대해 Transact-SQL 문을 실행하고 영향을 받는 행을 반환합니다.
                comm.ExecuteNonQuery()
            Catch exp As MySql.Data.MySqlClient.MySqlException
                exp.ToString()
                Return False
            End Try

            m_conn.Close()
            Return True
        End Function

        ' 연결된 데이터베이스를 Open 한다.
        Private Function Open() As Boolean
            If m_conn Is Nothing Then
                Return False
            End If

            Try
                ' 데이터베이스 Open
                m_conn.Open()
            Catch exp As MySql.Data.MySqlClient.MySqlException
                exp.ToString()
                Return False
            End Try

            Return True
        End Function

        ' 데이터베이스에 사원 정보 입력(성공 : 리턴값 True, 실패 : 리턴값 False
        Public Function AddData(ByVal emp As Employee) As Boolean
            If m_conn Is Nothing Then
                Return False
            End If

            ' 연결된 데이터베이스 Open
            Open()

            ' 사원 정보를 저장하기 위한 SQL 쿼리
            Dim strQuery As String = "INSERT INTO EMPLOYEE VALUES (@NAME, @AGE, @POSITION, @PART, @ENTERING)"

            ' 쿼리 객체 생성
            comm = New MySqlCommand(strQuery, m_conn)

            ' 필드 데이터 타입 설정
            comm.Parameters.Add("@NAME", MySqlDbType.VarChar)
            comm.Parameters.Add("@AGE", MySqlDbType.Int32)
            comm.Parameters.Add("@POSITION", MySqlDbType.VarChar)
            comm.Parameters.Add("@PART", MySqlDbType.VarChar)
            comm.Parameters.Add("@ENTERING", MySqlDbType.DateTime)

            ' 필드 값 설정
            comm.Parameters("@NAME").Value = emp.strName
            comm.Parameters("@AGE").Value = emp.iAge
            comm.Parameters("@POSITION").Value = emp.strPosition
            comm.Parameters("@PART").Value = emp.strPart
            comm.Parameters("@ENTERING").Value = emp.dDatetime

            ' 쿼리 실행
            Try
                If comm.ExecuteNonQuery() = 1 Then
                    comm.Dispose()

                    ' Close 메서드가 호출되어야 변경된 데이터베이스의 내용이 물리적인 파일에 기록된다. 
                    ' 따라서 장치(IEC-Series)의 전원이 임의로 Off될수있는 시스템에서는 
                    ' 데이터 베이스의 변경이 발생할 경우 m_conn.Close()메서드를 반드시 호출 하시기 바랍니다.
                    m_conn.Close()
                    Return True
                End If

            Catch ex As MySql.Data.MySqlClient.MySqlException
                ex.ToString()
                Return False
            End Try

            m_conn.Close()
            Return False
        End Function

        ' 데이터베이스에 사원 정보 삭제(성공 : 리턴값 True, 실패 : 리턴값 False)
        Public Function DeleteData(ByVal strName As String) As Boolean
            If m_conn Is Nothing Then
                Return False
            End If

            ' 연결된 데이터베이스 Open
            Open()

            ' 사원 정보를 삭제하기 위한 SQL 쿼리
            Dim strQuery As String = "DELETE FROM EMPLOYEE WHERE NAME = '" & strName & "'"
            ' 쿼리 객체 생성
            Dim comm As MySqlCommand = New MySqlCommand(strQuery, m_conn)
            ' 쿼리 실행
            If comm.ExecuteNonQuery() = 1 Then
                comm.Dispose()

                ' Close 메서드가 호출되어야 변경된 데이터베이스의 내용이 물리적인 파일에 기록된다. 
                ' 따라서 장치(IEC-Series)의 전원이 임의로 Off될수있는 시스템에서는 
                ' 데이터 베이스의 변경이 발생할 경우 m_conn.Close()메서드를 반드시 호출 하시기 바랍니다. 
                m_conn.Close()
                Return True
            End If

            Return False
        End Function

        ' 사원 정보를 검색 합니다. (인자가 * 인경우 모든 사원 정보를 리턴 합니다.) 
        Public Function SearchData(ByVal strName As String) As DataSet
            Dim retDataSet As DataSet = New DataSet()
            Dim strQuery As String

            ' 사원 정보를 얻기위한 위한 SQL 쿼리 (* 인경우 모든 정보를 얻음)
            If strName = "*" Then
                strQuery = "SELECT NAME, AGE, POSITION, PART, ENTERING FROM EMPLOYEE"
            Else
                strQuery = "SELECT NAME, AGE, POSITION, PART, ENTERING FROM EMPLOYEE WHERE NAME = '" & strName & "'"
            End If

            If m_conn Is Nothing Then
                Return Nothing
            End If

            ' 연결된 데이터베이스 Open
            Open()

            Dim adpt As MySqlDataAdapter = New MySqlDataAdapter(strQuery, m_conn)
            adpt.Fill(retDataSet)
            m_conn.Close()
            Return retDataSet
        End Function
    End Class
End Namespace
